Module ffi

Module ffi 

Structs

AccessClass
External data classes a chip may read ROM/RAM data from.
Chip
Opaque handle to a single emulated chip instance.
ChipType
Supported Yamaha FM/SSG chip families.
Fidelity
Sample-rate/accuracy tradeoff, via the ymfm opn_fidelity setting. Only meaningful for YM2203/YM2608/YM2610/YM2610B; a no-op elsewhere.

Functions

create_chip
Create a chip with all optional interface callbacks disabled.
create_chip_with_callbacks
Create a chip and forward ymfm interface callbacks to callbacks.