InterfaceCallbacks

Struct InterfaceCallbacks 

pub struct InterfaceCallbacks { /* private fields */ }
Opaque callback adapter owned by the native chip instance.

Implementations

Source

impl InterfaceCallbacks

pub fn new(handler: InterfaceHandler) -> Self

Create a callback adapter containing the supplied host handlers.

Examples found in repository?
examples/vgmrender.rs (lines 47-49)
39    fn new(chip_type: ChipType, clock: u32) -> Self {
40        let state = Rc::new(RefCell::new(VgmHandlerState {
41            data: std::array::from_fn(|_| Vec::new()),
42        }));
43        let pcm_offset = Rc::new(RefCell::new(0u32));
44        let chip = ffi::create_chip_with_callbacks(
45            chip_type,
46            clock,
47            Box::new(InterfaceCallbacks::new(vgm_handler_with_state(Rc::clone(
48                &state,
49            )))),
50        );
51        let channels = chip.channels() as usize;
52        let step: EmulatedTime = 0x1_0000_0000i64 / i64::from(chip.sample_rate());
53        Self {
54            chip,
55            channels,
56            queue: std::collections::VecDeque::new(),
57            pos: 0,
58            step,
59            native: vec![0i32; channels],
60            handler_state: state,
61            pcm_offset,
62        }
63    }